﻿using HeuristicLab.Clients.Hive.WebJobManager.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace HeuristicLab.Clients.Hive.WebJobManager.Services
{
    /// <summary>
    /// A service that will handle all the logins and will serve as a singleton that controls
    /// all the necessary Services used by Hive. (HiveServiceLocatorWeb and HiveClientWeb instances per user). 
    /// Users are identified with a seperate Guid made by the web version to ensure login security.
    /// </summary>
    public class WebLoginService
    {
        private static WebLoginService instance;
        private List<LoginViewModel> loggedIn;
        private List<HiveServiceLocatorWeb> locators;
        private List<HiveClientWeb> webclients;
        private List<FileOpeningService> fileopeners;
        /// <summary>
        /// Service instance that keeps all the information per user seperated. 
        /// </summary>
        public static WebLoginService Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new WebLoginService();
                }
                return instance;
            }
        }
        /// <summary>
        /// Initialize all lists
        /// </summary>
        public WebLoginService()
        {

            loggedIn = new List<LoginViewModel>();
            locators = new List<HiveServiceLocatorWeb>();
            webclients = new List<HiveClientWeb>();
            fileopeners = new List<FileOpeningService>();
        }
        /// <summary>
        /// Creates a new entry for a user. Services will be kept running here untill the user logs out or times out. 
        /// </summary>
        /// <param name="log">LoginViewModel from new user</param>
        /// <param name="loc">HiveServiceLocatorWeb created during login</param>
        public void newLogin(LoginViewModel log, HiveServiceLocatorWeb loc)
        {
            destroyPossiblePreviousSessions(log.loginName);
            var ind = loggedIn.Find(x => x.loginName == log.loginName);
            if (ind != null)
            {
                logout(ind.userId);
            }
            loggedIn.Add(log);
            locators.Add(loc);
            var temp = new HiveClientWeb(loc, log.userId);
            webclients.Add(temp);
            var temp2 = new FileOpeningService(log.userId);
            fileopeners.Add(temp2);
        }

        private void destroyPossiblePreviousSessions(string client)
        {
            var ind = loggedIn.FindAll(x => x.loginName == client);
            foreach (var i in ind)
            {
                locators.RemoveAll(x => x.UserId == i.userId);
                webclients.RemoveAll(x => x.UserId == i.userId);
                fileopeners.RemoveAll(x => x.UserId == i.userId);
                loggedIn.RemoveAll(x => x.userId == i.userId);
            }

        }
        /// <summary>
        /// Returns the service locator for a user
        /// </summary>
        /// <param name="id">User id</param>
        /// <returns></returns>
        public HiveServiceLocatorWeb getServiceLocator(Guid id)
        {
            return locators.Find(x => x.UserId == id);
        }
        /// <summary>
        /// Returns HiveClientWeb for a user
        /// </summary>
        /// <param name="id">User id</param>
        /// <returns></returns>
        public HiveClientWeb getClientWeb(Guid id)
        {
            return webclients.Find(x => x.UserId == id);
        }
        /// <summary>
        /// Returns the LoginViewModel for a user
        /// </summary>
        /// <param name="id">User id</param>
        /// <returns></returns>
        public LoginViewModel getLoginViewModel(Guid id)
        {
            return loggedIn.Find(x => x.userId == id);
        }
        /// <summary>
        /// Returns the FileOpenerService for the user
        /// </summary>
        /// <param name="id">User id</param>
        /// <returns></returns>
        public FileOpeningService getFileOpener(Guid id)
        {
            return fileopeners.Find(x => x.UserId == id);
        }
        /// <summary>
        /// Removes all traces from the logged in user
        /// </summary>
        /// <param name="id">User id</param>
        public void logout(Guid id)
        {
            loggedIn.RemoveAll(x => x.userId == id);
            webclients.RemoveAll(x => x.UserId == id);
            locators.RemoveAll(x => x.UserId == id);
            fileopeners.RemoveAll(x => x.UserId == id);
        }
    }
}
